// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the Peter Jodda license:
// Everyboy may use it on his own risk. This header must stay in the file!
// If you expand this file, please contact me, that I can integrate the
// enhancemants in the original file.
//

#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/soundcard.h>
#include <sys/ioctl.h>


#include "audioplayer.h"
#include "saveblock.h"

//===========================================================================
// constructor for the audio recorder
//===========================================================================

AudioPlayer::AudioPlayer(const char * cp_DeviceName)
{
   strncpy(m_ca_DeviceName,cp_DeviceName,99);
   m_ca_DeviceName[99] ='\0';

   i_Error =0;

   // Attempt to open the audio device for read only
   m_i_AudioHandle = open(m_ca_DeviceName, O_WRONLY  );
   if (m_i_AudioHandle == -1)
   {
      fprintf(stderr,"error open the device\n");
      i_Error = 1;
   }


   if (ioctl(m_i_AudioHandle, SNDCTL_DSP_GETFMTS, &i_AudioFormatMask) < 0) 
   {
      i_Error =2;
   }

//   pc_SaveBlockArray = NULL;
   
   
   
} // AudioPlayer()

//===========================================================================
//
//===========================================================================

int AudioPlayer::SetDevice(const char * cp_DeviceName)
{
   i_Error =0;

   strncpy(m_ca_DeviceName,cp_DeviceName,99);
   m_ca_DeviceName[99] ='\0';

   // if opened before, close it
   if(m_i_AudioHandle >=0)
   {
      close(m_i_AudioHandle);
      m_i_AudioHandle = -1;
   }

   // Attempt to open the audio device for read only
   m_i_AudioHandle = open(m_ca_DeviceName, O_WRONLY  );
   if (m_i_AudioHandle == -1)
   {
      fprintf(stderr,"error open the device\n");
      i_Error = 1;
   }


   if (ioctl(m_i_AudioHandle, SNDCTL_DSP_GETFMTS, &i_AudioFormatMask) < 0) 
   {
      i_Error =2;
   }

   return i_Error;
} // AudioRecorder()


//===========================================================================
// destructor
//===========================================================================

AudioPlayer::~AudioPlayer()
{
   if(m_i_AudioHandle>0)
   {
      close(m_i_AudioHandle);
   }
} // ~AudioPlayer()

//===========================================================================
// the the sampling speed
//===========================================================================


bool AudioPlayer::SetSpeed(int speed)
{
   int i_Bits;
   int i_Stereo;
   
   i_Bits   = 16;
   i_Stereo = 2;


   if(m_i_AudioHandle <0)
   {
      return false;
   }

   SyncAudio();

   /* Set the sample speed, size and stereoness */
   if (ioctl(m_i_AudioHandle, SNDCTL_DSP_SAMPLESIZE, &i_Bits) < 0)
   {
      return false;
   }

   if (ioctl(m_i_AudioHandle, SNDCTL_DSP_STEREO, &i_Stereo) < 0)
   {
      return false;
   }

   if (ioctl(m_i_AudioHandle, SNDCTL_DSP_SPEED, &speed) < 0)
   {
      return false;
   }

   return true;


} // SetSpeed() 

//===========================================================================
// sync the audio card
//===========================================================================

    
bool AudioPlayer::SyncAudio()
{
   if(m_i_AudioHandle <0)
   {
      return false;
   }

    if (ioctl (m_i_AudioHandle, SNDCTL_DSP_SYNC, NULL) < 0)
    {
    
	return false;
    }
    return true;
    
} // SyncAudio()

//===========================================================================
// play some date from the device
//===========================================================================

int AudioPlayer::PlayAudio(unsigned char *puc_Buffer, int i_BufferLength)
{
   int count;


   if(m_i_AudioHandle <0)
   {
      return 0;
   }

   // do the real write
   count = write(m_i_AudioHandle, 
                   puc_Buffer,
                   i_BufferLength);

  
   return count;
} // PlayAudio()


//===========================================================================
//
//===========================================================================





